@@ -459,7 +459,9 @@ func addCheckVarType(V, T string, pos token.Pos) {
 }
 
 func addCheckFieldType(S, F, T string, opt bool, pos token.Pos) {
-	checkFieldTypeRules[S] = append(checkFieldTypeRules[S], &checkFieldType{S, F, T, opt, pos})
+	if !strings.Contains(S, "|") {
+		checkFieldTypeRules[S] = append(checkFieldTypeRules[S], &checkFieldType{S, F, T, opt, pos})
+	}
 }
 
 func addCheckConstVal(C string, V constant.Value, pos token.Pos) {
@@ -547,7 +549,6 @@ func check() {
 	allok := true
 
 	for _, rule := range checkVarTypeRules {
-		//TODO: implement
 		pos := fset.Position(rule.pos)
 		def := lookupPackage(pkgmap, "runtime").Types.Scope().Lookup(rule.V)
 		if def == nil {
@@ -618,7 +619,7 @@ func check() {
 	for _, C := range Cs {
 		rules := checkConstValRules[C]
 		pos := fset.Position(rules[0].pos)
-		def := lookupPackage(pkgmap, "runtime").Types.Scope().Lookup(C)
+		def := findConst(pkgmap, C)
 		if def == nil {
 			fmt.Fprintf(os.Stderr, "%s:%d: could not find constant %s\n", pos.Filename, pos.Line, C)
 			allok = false
@@ -654,6 +655,21 @@ func fieldTypeByName(typ *types.Struct, name string) types.Type {
 	return nil
 }
 
+func findConst(pkgmap map[string]*packages.Package, Cs string) types.Object {
+	for _, C := range strings.Split(Cs, "|") {
+		pkg := lookupPackage(pkgmap, "runtime")
+		if dot := strings.Index(C, "."); dot >= 0 {
+			pkg = lookupPackage(pkgmap, C[:dot])
+			C = C[dot+1:]
+		}
+		def := pkg.Types.Scope().Lookup(C)
+		if def != nil {
+			return def
+		}
+	}
+	return nil
+}
+
 func matchType(typ types.Type, T string) bool {
 	if T == "anytype" {
 		return true
