@@ -36,13 +36,14 @@ func I386Arch(goos string) *Arch {
 		SPRegNum:                         regnum.I386_Esp,
 		asmRegisters:                     i386AsmRegisters,
 		RegisterNameToDwarf:              nameToDwarfFunc(regnum.I386NameToDwarf),
+		RegnumToString:                   regnum.I386ToName,
 	}
 }
 
 func i386FixFrameUnwindContext(fctxt *frame.FrameContext, pc uint64, bi *BinaryInfo) *frame.FrameContext {
 	i := bi.Arch
 	if i.sigreturnfn == nil {
-		i.sigreturnfn = bi.LookupFunc["runtime.sigreturn"]
+		i.sigreturnfn = bi.lookupOneFunc("runtime.sigreturn")
 	}
 
 	if fctxt == nil || (i.sigreturnfn != nil && pc >= i.sigreturnfn.Entry && pc < i.sigreturnfn.End) {
@@ -67,15 +68,15 @@ func i386FixFrameUnwindContext(fctxt *frame.FrameContext, pc uint64, bi *BinaryI
 		return &frame.FrameContext{
 			RetAddrReg: regnum.I386_Eip,
 			Regs: map[uint64]frame.DWRule{
-				regnum.I386_Eip: frame.DWRule{
+				regnum.I386_Eip: {
 					Rule:   frame.RuleOffset,
 					Offset: int64(-i.PtrSize()),
 				},
-				regnum.I386_Ebp: frame.DWRule{
+				regnum.I386_Ebp: {
 					Rule:   frame.RuleOffset,
 					Offset: int64(-2 * i.PtrSize()),
 				},
-				regnum.I386_Esp: frame.DWRule{
+				regnum.I386_Esp: {
 					Rule:   frame.RuleValOffset,
 					Offset: 0,
 				},
@@ -89,7 +90,7 @@ func i386FixFrameUnwindContext(fctxt *frame.FrameContext, pc uint64, bi *BinaryI
 	}
 
 	if i.crosscall2fn == nil {
-		i.crosscall2fn = bi.LookupFunc["crosscall2"]
+		i.crosscall2fn = bi.lookupOneFunc("crosscall2")
 	}
 
 	// TODO(chainhelen), need to check whether there is a bad frame descriptor like amd64.
@@ -117,16 +118,28 @@ func i386FixFrameUnwindContext(fctxt *frame.FrameContext, pc uint64, bi *BinaryI
 // SwitchStack will use the current frame to determine if it's time to
 func i386SwitchStack(it *stackIterator, _ *op.DwarfRegisters) bool {
 	if it.frame.Current.Fn == nil {
+		if it.systemstack && it.g != nil && it.top {
+			it.switchToGoroutineStack()
+			return true
+		}
 		return false
 	}
 	switch it.frame.Current.Fn.Name {
 	case "runtime.asmcgocall", "runtime.cgocallback_gofunc": // TODO(chainhelen), need to support cgo stacktraces.
 		return false
-	case "runtime.goexit", "runtime.rt0_go", "runtime.mcall":
+	case "runtime.goexit", "runtime.rt0_go":
 		// Look for "top of stack" functions.
 		it.atend = true
 		return true
 
+	case "runtime.mcall":
+		if it.systemstack && it.g != nil {
+			it.switchToGoroutineStack()
+			return true
+		}
+		it.atend = true
+		return true
+
 	case "runtime.mstart":
 		// Calls to runtime.systemstack will switch to the systemstack then:
 		// 1. alter the goroutine stack so that it looks like systemstack_switch
@@ -147,25 +160,16 @@ func i386SwitchStack(it *stackIterator, _ *op.DwarfRegisters) bool {
 		it.switchToGoroutineStack()
 		return true
 
-	default:
-		if it.systemstack && it.top && it.g != nil && strings.HasPrefix(it.frame.Current.Fn.Name, "runtime.") && it.frame.Current.Fn.Name != "runtime.fatalthrow" {
-			// The runtime switches to the system stack in multiple places.
-			// This usually happens through a call to runtime.systemstack but there
-			// are functions that switch to the system stack manually (for example
-			// runtime.morestack).
-			// Since we are only interested in printing the system stack for cgo
-			// calls we switch directly to the goroutine stack if we detect that the
-			// function at the top of the stack is a runtime function.
-			//
-			// The function "runtime.fatalthrow" is deliberately excluded from this
-			// because it can end up in the stack during a cgo call and switching to
-			// the goroutine stack will exclude all the C functions from the stack
-			// trace.
+	case "runtime.newstack", "runtime.systemstack":
+		if it.systemstack && it.g != nil {
 			it.switchToGoroutineStack()
 			return true
 		}
 
 		return false
+
+	default:
+		return false
 	}
 }
 
@@ -204,7 +208,7 @@ func i386AddrAndStackRegsToDwarfRegisters(staticBase, pc, sp, bp, lr uint64) op.
 }
 
 func i386DwarfRegisterToString(j int, reg *op.DwarfRegister) (name string, floatingPoint bool, repr string) {
-	name = regnum.I386ToName(j)
+	name = regnum.I386ToName(uint64(j))
 
 	if reg == nil {
 		return name, false, ""
