@@ -494,7 +494,7 @@ func (g *G) Defer() *Defer {
 		return nil
 	}
 	d := &Defer{variable: dvar}
-	d.load()
+	d.load(true)
 	return d
 }
 
@@ -930,11 +930,11 @@ func (v *Variable) parseG() (*G, error) {
 	}
 
 	status := uint64(0)
-	if atomicStatus := v.loadFieldNamed("atomicstatus"); /* +rtype uint32|runtime/internal/atomic.Uint32 */ atomicStatus != nil {
+	if atomicStatus := v.loadFieldNamed("atomicstatus"); /* +rtype uint32|runtime/internal/atomic.Uint32|internal/runtime/atomic.Uint32 */ atomicStatus != nil {
 		if constant.Val(atomicStatus.Value) != nil {
 			status, _ = constant.Uint64Val(atomicStatus.Value)
 		} else {
-			atomicStatus := atomicStatus              // +rtype runtime/internal/atomic.Uint32
+			atomicStatus := atomicStatus              // +rtype runtime/internal/atomic.Uint32|internal/runtime/atomic.Uint32
 			vv := atomicStatus.fieldVariable("value") // +rtype uint32
 			if vv == nil {
 				unreadable = true
@@ -1169,7 +1169,7 @@ func (v *Variable) structMember(memberName string) (*Variable, error) {
 func readVarEntry(entry *godwarf.Tree, image *Image) (name string, typ godwarf.Type, err error) {
 	name, ok := entry.Val(dwarf.AttrName).(string)
 	if !ok {
-		return "", nil, fmt.Errorf("malformed variable DIE (name)")
+		return "", nil, errors.New("malformed variable DIE (name)")
 	}
 
 	typ, err = entry.Type(image.dwarf, image.index, image.typeCache)
@@ -1389,8 +1389,15 @@ func (v *Variable) loadValueInternal(recurseLevel int, cfg LoadConfig) {
 					break
 				}
 				f, _ := v.toField(field)
+				f.Name = field.Name
+				if t.StructName == "" && len(f.Name) > 0 && f.Name[0] == '&' && f.Kind == reflect.Ptr {
+					// This struct is a closure struct and the field is actually a variable
+					// captured by reference.
+					f = f.maybeDereference()
+					f.Flags |= VariableEscaped
+					f.Name = field.Name[1:]
+				}
 				v.Children = append(v.Children, *f)
-				v.Children[i].Name = field.Name
 				v.Children[i].loadValueInternal(recurseLevel+1, cfg)
 			}
 		}
@@ -1435,7 +1442,7 @@ func (v *Variable) loadValueInternal(recurseLevel int, cfg LoadConfig) {
 			v.FloatSpecial = FloatIsNaN
 		}
 	case reflect.Func:
-		v.readFunctionPtr()
+		v.loadFunctionPtr(recurseLevel, cfg)
 	default:
 		v.Unreadable = fmt.Errorf("unknown or unsupported kind: %q", v.Kind.String())
 	}
@@ -1472,7 +1479,7 @@ func convertToEface(srcv, dstv *Variable) error {
 
 func readStringInfo(mem MemoryReadWriter, arch *Arch, addr uint64, typ *godwarf.StringType) (uint64, int64, error) {
 	// string data structure is always two ptrs in size. Addr, followed by len
-	// http://research.swtch.com/godata
+	// https://research.swtch.com/godata
 
 	mem = cacheMemory(mem, addr, arch.PtrSize()*2)
 
@@ -1833,7 +1840,7 @@ func (v *Variable) readFloatRaw(size int64) (float64, error) {
 		return n, nil
 	}
 
-	return 0.0, fmt.Errorf("could not read float")
+	return 0.0, errors.New("could not read float")
 }
 
 func (v *Variable) writeFloatRaw(f float64, size int64) error {
@@ -1915,7 +1922,7 @@ func (v *Variable) writeCopy(srcv *Variable) error {
 	return err
 }
 
-func (v *Variable) readFunctionPtr() {
+func (v *Variable) loadFunctionPtr(recurseLevel int, cfg LoadConfig) {
 	// dereference pointer to find function pc
 	v.closureAddr = v.funcvalAddr()
 	if v.Unreadable != nil {
@@ -1941,6 +1948,14 @@ func (v *Variable) readFunctionPtr() {
 	}
 
 	v.Value = constant.MakeString(fn.Name)
+	cst := fn.extra(v.bi).closureStructType
+	v.Len = int64(len(cst.Field))
+
+	if recurseLevel <= cfg.MaxVariableRecurse {
+		v2 := v.newVariable("", v.closureAddr, cst, v.mem)
+		v2.loadValueInternal(recurseLevel, cfg)
+		v.Children = v2.Children
+	}
 }
 
 // funcvalAddr reads the address of the funcval contained in a function variable.
@@ -1966,7 +1981,7 @@ func (v *Variable) loadMap(recurseLevel int, cfg LoadConfig) {
 
 	for skip := 0; skip < v.mapSkip; skip++ {
 		if ok := it.next(); !ok {
-			v.Unreadable = fmt.Errorf("map index out of bounds")
+			v.Unreadable = errors.New("map index out of bounds")
 			return
 		}
 	}
@@ -2028,7 +2043,7 @@ func (v *Variable) mapIterator() *mapIterator {
 
 	maptype, ok := sv.RealType.(*godwarf.StructType)
 	if !ok {
-		v.Unreadable = fmt.Errorf("wrong real type for map")
+		v.Unreadable = errors.New("wrong real type for map")
 		return nil
 	}
 
@@ -2169,7 +2184,7 @@ func (it *mapIterator) nextBucket() bool {
 
 	// sanity checks
 	if it.tophashes == nil || it.keys == nil || it.values == nil {
-		it.v.Unreadable = fmt.Errorf("malformed map type")
+		it.v.Unreadable = errors.New("malformed map type")
 		return false
 	}
 
@@ -2272,7 +2287,7 @@ func (v *Variable) readInterface() (_type, data *Variable, isnil bool) {
 
 	ityp := resolveTypedef(&v.RealType.(*godwarf.InterfaceType).TypedefType).(*godwarf.StructType)
 
-	// +rtype -field iface.tab *itab
+	// +rtype -field iface.tab *itab|*internal/abi.ITab
 	// +rtype -field iface.data unsafe.Pointer
 	// +rtype -field eface._type *_type|*internal/abi.Type
 	// +rtype -field eface.data unsafe.Pointer
@@ -2280,15 +2295,18 @@ func (v *Variable) readInterface() (_type, data *Variable, isnil bool) {
 	for _, f := range ityp.Field {
 		switch f.Name {
 		case "tab": // for runtime.iface
-			tab, _ := v.toField(f) // +rtype *itab
+			tab, _ := v.toField(f) // +rtype *itab|*internal/abi.ITab
 			tab = tab.maybeDereference()
 			isnil = tab.Addr == 0
 			if !isnil {
 				var err error
-				_type, err = tab.structMember("_type") // +rtype *_type|*internal/abi.Type
+				_type, err = tab.structMember("Type") // +rtype *internal/abi.Type
 				if err != nil {
-					v.Unreadable = fmt.Errorf("invalid interface type: %v", err)
-					return
+					_type, err = tab.structMember("_type") // +rtype *_type|*internal/abi.Type
+					if err != nil {
+						v.Unreadable = fmt.Errorf("invalid interface type: %v", err)
+						return
+					}
 				}
 			}
 		case "_type": // for runtime.eface
@@ -2315,11 +2333,17 @@ func (v *Variable) loadInterface(recurseLevel int, loadData bool, cfg LoadConfig
 	}
 
 	if data == nil {
-		v.Unreadable = fmt.Errorf("invalid interface type")
+		v.Unreadable = errors.New("invalid interface type")
 		return
 	}
 
-	typ, kind, err := runtimeTypeToDIE(_type, data.Addr)
+	mds, err := LoadModuleData(_type.bi, _type.mem)
+	if err != nil {
+		v.Unreadable = fmt.Errorf("error loading module data: %v", err)
+		return
+	}
+
+	typ, kind, err := RuntimeTypeToDIE(_type, data.Addr, mds)
 	if err != nil {
 		v.Unreadable = err
 		return
