@@ -901,7 +901,7 @@ func TestPrintContextParkedGoroutine(t *testing.T) {
 		frameout := strings.Split(term.MustExec("frame 0"), "\n")
 		t.Logf("frame 0 -> %q", frameout)
 		if strings.Contains(frameout[0], "stacktraceme") {
-			t.Fatal("bad output for `frame 0` command on a parked goorutine")
+			t.Fatal("bad output for `frame 0` command on a parked goroutine")
 		}
 
 		listout := strings.Split(term.MustExec("list"), "\n")
@@ -1366,7 +1366,7 @@ func TestTranscript(t *testing.T) {
 	withTestTerminal("math", t, func(term *FakeTerminal) {
 		term.MustExec("break main.main")
 		out := term.MustExec("continue")
-		if !strings.HasPrefix(out, "> main.main()") {
+		if !strings.HasPrefix(out, "> [Breakpoint 1] main.main()") {
 			t.Fatalf("Wrong output for next: <%s>", out)
 		}
 		fh, err := os.CreateTemp("", "test-transcript-*")
@@ -1435,6 +1435,47 @@ func TestCreateBreakpointByLocExpr(t *testing.T) {
 	})
 }
 
+func TestCreateBreakpointWithCondition(t *testing.T) {
+	withTestTerminal("break", t, func(term *FakeTerminal) {
+		term.MustExec("break bp1 main.main:4 if i == 3")
+		out := term.MustExec("breakpoints")
+		if !strings.Contains(out, "Breakpoint bp1") {
+			t.Fatal("incorrect breakpoint name")
+		}
+		listIsAt(t, term, "continue", 7, -1, -1)
+		out = term.MustExec("print i")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "3\n") {
+			t.Fatalf("wrong value of i")
+		}
+	})
+}
+
+func TestCreateBreakpointWithCondition2(t *testing.T) {
+	withTestTerminal("break", t, func(term *FakeTerminal) {
+		term.MustExec("continue main.main:4")
+		term.MustExec("break if i == 3")
+		out := term.MustExec("breakpoints")
+		if strings.Contains(out, "Breakpoint if") {
+			t.Fatal("incorrect breakpoint name, should be ID")
+		}
+		listIsAt(t, term, "continue", 7, -1, -1)
+		out = term.MustExec("print i")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "3\n") {
+			t.Fatalf("wrong value of i")
+		}
+	})
+}
+
+func TestCreateBreakpointWithCondition3(t *testing.T) {
+	withTestTerminal("test if path/main", t, func(term *FakeTerminal) {
+		// We should not attempt to parse this as a condition.
+		term.MustExec(`break _fixtures/test if path/main.go:4`)
+		listIsAt(t, term, "continue", 4, -1, -1)
+	})
+}
+
 func TestRestartBreakpoints(t *testing.T) {
 	// Tests that breakpoints set using just a line number and with a line
 	// offset are preserved after restart. See issue #3423.
