@@ -127,7 +127,7 @@ type CommandOut struct {
 
 // Command interrupts, continues and steps through the program.
 func (s *RPCServer) Command(command api.DebuggerCommand, cb service.RPCCallback) {
-	st, err := s.debugger.Command(&command, cb.SetupDoneChan())
+	st, err := s.debugger.Command(&command, cb.SetupDoneChan(), cb.DisconnectChan())
 	if err != nil {
 		cb.Return(nil, err)
 		return
@@ -578,7 +578,8 @@ func (s *RPCServer) ListSources(arg ListSourcesIn, out *ListSourcesOut) error {
 }
 
 type ListFunctionsIn struct {
-	Filter string
+	Filter      string
+	FollowCalls int
 }
 
 type ListFunctionsOut struct {
@@ -587,7 +588,7 @@ type ListFunctionsOut struct {
 
 // ListFunctions lists all functions in the process matching filter.
 func (s *RPCServer) ListFunctions(arg ListFunctionsIn, out *ListFunctionsOut) error {
-	fns, err := s.debugger.Functions(arg.Filter)
+	fns, err := s.debugger.Functions(arg.Filter, arg.FollowCalls)
 	if err != nil {
 		return err
 	}
@@ -978,7 +979,7 @@ type ExaminedMemoryOut struct {
 
 func (s *RPCServer) ExamineMemory(arg ExamineMemoryIn, out *ExaminedMemoryOut) error {
 	if arg.Length > 1000 {
-		return fmt.Errorf("len must be less than or equal to 1000")
+		return errors.New("len must be less than or equal to 1000")
 	}
 	Mem, err := s.debugger.ExamineMemory(arg.Address, arg.Length)
 	if err != nil {
